import daj.*;

/**
 *	Algorithms for Distributed Systems, SS 00, Project)
 *
 *	Dijkstra-Scholten algorithm message. Can be an acknowledgement
 *	of Dijkstra-Scholten algorithm or a message of monitored algorithm.
 *	Sender identification is contained for informational purposes.
 *
 *	@author Jrgen Hartl, 9756179 / 881
 *	@author Rene Kobler, 9655888 / 881
 *	@author Christian Gintenreiter, 9655980 / 881
 *	@version 1.0
 */
class DijkstraScholtenMessage extends Message
{
	/**
	 *	Constant defines acknowledgement message type.
	 */
	protected static final int ACKNOWLEDGEMENT = 0;

	/**
	 *	Constant defines monitored algorithm message type.
	 */
	protected static final int ALGORITHM = 1;

	/**
	 *	Constant defines message type descriptions.
	 */
	protected static final String[] TYPEDESCRIPTION = {
		new String("Acknowledgement"),
		new String("Algorithm message")
	};

	/**
	 *	Holds sender identification for informational purposes.
	 */
	String senderId;

	/**
	 *	Holds message type.
	 */
	int type;

	/**
	 *	Constructor.
	 *	Message is a algorithm message with unknown sender by default.
	 */
	public DijkstraScholtenMessage()
	{
		type = ALGORITHM;
		senderId = new String("(unknown)");
	}

	/**
	 *	Constructor.
	 *
	 *	@param type Type of message (acknowledgement or algorithm).
	 *	@param senderId	Message sender identification.
	 */
	public DijkstraScholtenMessage(int type, String senderId)
	{
		this.type = type;
		this.senderId = senderId;
	}

	/**
	 *	Sets sender identification.
	 *
	 *	@param senderId Sender identification.
	 */
	public void setSenderId(String senderId)
	{
		this.senderId = senderId;
	}

	/**
	 *	Indicates that message is an acknowledgement.
	 *
	 *	@return True if message is an acknowledgement.
	 */
	public boolean isAcknowledgement()
	{
		return (type == ACKNOWLEDGEMENT);
	}

	/**
	 *	Indicates that message is a monitored algorithm message.
	 *
	 *	@return True if message is a monitored algorithm message.
	 */
	public boolean isAlgorithmMessage()
	{
		return (type == ALGORITHM);
	}

	/**
	 *	Returns the sender identification.
	 *
	 *	@return The sender identification.
	 */
	public String getSenderId()
	{
		return senderId;
	}

	/**
	 *	Returns description of the message.
	 *
	 *	@return Description of the message.
	 */
	public String getText()
	{
		return "Dijkstra-Scholten message:\n" + TYPEDESCRIPTION[type] + "\nSender: " + senderId;
	}
}
