import daj.*;
import java.util.Random;

/**
 *	Algorithms for Distributed Systems, SS 00, Project)
 *
 *	Main application for following example:
 *	Dijkstra-Scholten algorithm monitoring termination
 *	of AsynchBFS algorithm.
 *
 *	@author Jrgen Hartl, 9756179 / 881
 *	@author Rene Kobler, 9655888 / 881
 *	@author Christian Gintenreiter, 9655980 / 881
 *	@version 1.0
 */
public class Main extends Application
{
	/**
	 *	Constant defines number of processes (nodes).
	 */
	public static final int PROCESSCNT = 6;

	/**
	 *	Holds random generator.
	 */
	public static Random random = new Random();

	/**
	 *	Main function contructs application object.
	 *
	 *	@param args Command line arguments.
	 */
	public static void main(String[] args)
	{
		int seed;

		if (args.length >= 1)
		{
			try
			{
				seed = Integer.parseInt(args[0]);
				random = new Random(seed);
			}
			catch(Exception e)
			{
			}
		}
		new Main().run();
	}

	/**
	 *	Constructor.
	 */
	public Main()
	{
		super("AsynchBFS algorithm monitored by Dijkstra-Scholten termination detection algorithm", 400, 400);
	}

	/**
	 *	Constructs network and initializes nodes.
	 */
	public void construct()
	{
		Node[] nodes = new Node[PROCESSCNT];
		DijkstraScholtenProgram program;
		int[][] pos = {
			{ 100, 100 },
			{ 200, 100 },
			{ 250, 200 },
			{ 200, 300 },
			{ 100, 300 },
			{  50, 200 }};
		int i, j;

		for (i = 0; i < nodes.length; i++)
		{
			program = new AsynchBFSProgram(i);
			program.setNodeId(Integer.toString(i));
			/*
				each node must store info about partner node indizes and
				channel index vs. node index mapping
				(needed for DijkstraScholtenAssertion checks)
			*/
			program.initConnectionInfo(nodes.length - 1, nodes.length);
			nodes[i] = node(program, Integer.toString(i), pos[i][0], pos[i][1]);
		}
		for (i = 0; i < nodes.length - 1; i++)
		{
			for (j = i + 1; j < nodes.length; j++)
			{
				// store channel index vs. node index map entry
				((DijkstraScholtenProgram)nodes[i].getProgram()).setConnectionInfo(
					nodes[i].getOut().getSize(), j);
				((DijkstraScholtenProgram)nodes[j].getProgram()).setConnectionInfo(
					nodes[j].getOut().getSize(), i);
				link(nodes[i], nodes[j]);
				link(nodes[j], nodes[i]);
			}
		}
		for (i = 0; i < nodes.length; i++)
		{
			/*
				nodes must be initializes before any main-function is run
				to ensure the DijkstraScholtenAssertion holds
			*/
			((DijkstraScholtenProgram)nodes[i].getProgram()).init();
		}
	}

	/**
	 *	Returns description of application.
	 *
	 *	@return Description of application.
	 */
	public String getText()
	{
		return "AsynchBFS algorithm monitored by Dijkstra-Scholten termination detection algorithm";
	}
}
